@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 import os
 import re
 import requests
@@ -8,21 +6,24 @@ import time
 import xml.etree.ElementTree as ET
 import urllib.parse
 from collections import namedtuple
+from pathlib import Path
+from typing import Dict, Iterable, MutableMapping, NewType, Optional, Set, Tuple, Union
 
 from dateutil.parser import parse as parsedate
 from docutils import nodes, utils
 from sphinx.util.nodes import split_explicit_title
-from sphinx.util.console import bold, standout
+from sphinx.util.console import bold, standout  # type: ignore  # These are not explicitly exported as functions
 from sphinx import __version__ as sphinx_version
 
 if sphinx_version >= '1.6.0':
     from sphinx.util.logging import getLogger
 
-from ..doxylink import __version__
+from . import __version__
 from .parsing import normalise, ParseException
 
 Entry = namedtuple('Entry', ['kind', 'file'])
 
+Url = NewType("Url", str)
 
 def report_info(env, msg, docname=None, lineno=None):
     '''Convenience function for logging an informational
@@ -59,7 +60,7 @@ def report_warning(env, msg, docname=None, lineno=None):
     else:
         env.warn(docname, msg, lineno=lineno)
 
-def is_url(str_to_validate):
+def is_url(str_to_validate: str) -> bool:
     ''' Helper function to check if string contains URL
 
     Args:
@@ -82,7 +83,7 @@ class FunctionList:
     """A FunctionList maps argument lists to specific entries"""
     def __init__(self):
         self.kind = 'function_list'
-        self._arglist = {}  # type: MutableMapping[str, str]
+        self._arglist: MutableMapping[str, str] = {}
 
     def __getitem__(self, arglist: str) -> Entry:
         # If the user has requested a specific function through specifying an arglist then get the right anchor
@@ -101,6 +102,9 @@ class FunctionList:
     def add_overload(self, arglist: str, file: str) -> None:
         self._arglist[arglist] = file
 
+    def __repr__(self):
+        return f"FunctionList({self._arglist})"
+
 
 class SymbolMap:
     """A SymbolMap maps symbols to Entries or FunctionLists"""
@@ -153,7 +157,7 @@ class SymbolMap:
         return entry
 
 
-def parse_tag_file(doc: ET.ElementTree) -> dict:
+def parse_tag_file(doc: ET.ElementTree) -> Dict[str, Union[Entry, FunctionList]]:
     """
     Takes in an XML tree from a Doxygen tag file and returns a dictionary that looks something like:
 
@@ -177,7 +181,7 @@ def parse_tag_file(doc: ET.ElementTree) -> dict:
     :return: a dictionary mapping fully qualified symbols to files
     """
 
-    mapping = {}  # type: MutableMapping[str, Union[Entry, FunctionList]]
+    mapping: Dict[str, Union[Entry, FunctionList]] = {}
     function_list = []  # This is a list of function to be parsed and inserted into mapping at the end of the function.
     for compound in doc.findall('./compound'):
         compound_kind = compound.get('kind')
@@ -187,6 +191,11 @@ def parse_tag_file(doc: ET.ElementTree) -> dict:
         compound_name = compound.findtext('name')
         compound_filename = compound.findtext('filename')
 
+        if compound_name is None:
+            raise KeyError(f"Compound does not have a name")
+        if compound_filename is None:
+            raise KeyError(f"Compound {compound_name} does not have a filename")
+
         # TODO The following is a hack bug fix I think
         # Doxygen doesn't seem to include the file extension to <compound kind="file"><filename> entries
         # If it's a 'file' type, check if it _does_ have an extension, if not append '.html'
@@ -200,31 +209,36 @@ def parse_tag_file(doc: ET.ElementTree) -> dict:
             # If the member doesn't have an <anchorfile> element, use the parent compounds <filename> instead
             # This is the way it is in the qt.tag and is perhaps an artefact of old Doxygen
             anchorfile = member.findtext('anchorfile') or compound_filename
-            member_symbol = compound_name + '::' + member.findtext('name')
+            member_name = member.findtext('name')
+            if member_name is None:
+                raise KeyError(f"Member of {compound_name} does not have a name")
+            member_symbol = compound_name + '::' + member_name
             member_kind = member.get('kind')
             arglist_text = member.findtext('./arglist')  # If it has an <arglist> then we assume it's a function. Empty <arglist> returns '', not None. Things like typedefs and enums can have empty arglists
 
-            if arglist_text and member_kind not in {'variable', 'typedef', 'enumeration'}:
+            if arglist_text and member_kind not in {'variable', 'typedef', 'enumeration', "enumvalue"}:
                 function_list.append((member_symbol, arglist_text, member_kind, join(anchorfile, '#', member.findtext('anchor'))))
             else:
+                # Put the simple things directly into the mapping
                 mapping[member_symbol] = Entry(kind=member.get('kind'), file=join(anchorfile, '#', member.findtext('anchor')))
 
     for member_symbol, arglist, kind, anchor_link in function_list:
         try:
             normalised_arglist = normalise(member_symbol + arglist)[1]
         except ParseException as e:
-            print('Skipping %s %s%s. Error reported from parser was: %s' % (kind, member_symbol, arglist, e))
+            print(f'Skipping {kind} {member_symbol}{arglist}. Error reported from parser was: {e}')
         else:
-            if mapping.get(member_symbol) and isinstance(mapping[member_symbol], FunctionList):
-                mapping[member_symbol].add_overload(normalised_arglist, anchor_link)
-            else:
+            if member_symbol not in mapping:
                 mapping[member_symbol] = FunctionList()
-                mapping[member_symbol].add_overload(normalised_arglist, anchor_link)
+            member_mapping = mapping[member_symbol]
+            if not isinstance(member_mapping, FunctionList):
+                raise RuntimeError(f"Cannot add override to non-function '{member_symbol}'")
+            member_mapping.add_overload(normalised_arglist, anchor_link)
 
     return mapping
 
 
-def match_piecewise(candidates: set, symbol: str, sep: str='::') -> set:
+def match_piecewise(candidates: Iterable[str], symbol: str, sep: str='::') -> set:
     """
     Match the requested symbol reverse piecewise (split on ``::``) against the candidates.
     This allows you to under-specify the base namespace so that ``"MyClass"`` can match ``my_namespace::MyClass``
@@ -325,7 +339,7 @@ def create_role(app, tag_filename, rootdir, cache_name, pdf=""):
         try:
             url = app.env.doxylink_cache[cache_name]['mapping'][part]
         except LookupError as error:
-            inliner.reporter.warning('Could not find match for `%s` in `%s` tag file. Error reported was %s' % (part, tag_filename, error), line=lineno)
+            inliner.reporter.warning(f'Could not find match for `{part}` in `{tag_filename}` tag file. Error reported was {error}', line=lineno)
             return [nodes.inline(title, title)], []
         except ParseException as error:
             inliner.reporter.warning('Error while parsing `%s`. Is not a well-formed C++ function call or symbol.'
@@ -355,17 +369,17 @@ def create_role(app, tag_filename, rootdir, cache_name, pdf=""):
     return find_doxygen_link
 
 
-def extract_configuration(values):
+def extract_configuration(values: tuple) -> Tuple[Union[Url, Path], Optional[Union[Url, Path]], Optional[Union[Url, Path]]]:
     if len(values) == 3:
         tag_filename, rootdir, pdf_filename = values
     elif len(values) == 2:
         tag_filename = values[0]
         if values[1].endswith('.pdf'):
             pdf_filename = values[1]
-            rootdir = ""
+            rootdir = None
         else:
             rootdir = values[1]
-            pdf_filename = ""
+            pdf_filename = None
     else:
         raise ValueError("Config variable `doxylink` is incorrectly configured. Expected a tuple with 2 to 3 "
                             "elements; got %s" % values)
@@ -408,7 +422,7 @@ def fetch_file(app, source, output_path):
                                     "config variable; got %r" % source))
 
 
-def process_configuration(app, tag_filename, rootdir, pdf_filename):
+def process_configuration(app, tag_filename: Union[Url, Path], rootdir: Optional[Union[Url, Path]], pdf_filename: Optional[Union[Url, Path]]):
     """Processes the configured values for ``doxylink`` and ``doxylink_pdf_files`` and warns about potential issues.
 
     The type of builder decides which values shall be used.
